/* ------------------------------------------------------------------------- */
/*
 *  socktcp.h
 *
 *  Copyright (c) 2004 - 2006 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Sun 19 Feb 2006 13:35:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_SOCKTCP_H
#define CLX_SOCKTCP_H

#include <string>
#include <unistd.h>
#include "socket.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_socket_tcpserver
	/* --------------------------------------------------------------------- */
	template <
		int Type = SOCK_STREAM,
		int Family = AF_INET
	>
	class basic_socket_tcpserver : public basic_socket<Type, Family> {
	public:
		typedef basic_socket<Type, Family> socktcp;
		typedef basic_socket_address<Family> socket_address;
		
		// constructor and destructor
		basic_socket_tcpserver() : socktcp() {}
		
		explicit basic_socket_tcpserver(int port, int n = 100) :
			socktcp() { this->open(port, n); }
		
		virtual ~basic_socket_tcpserver() {}
		
		// operator
		virtual socket_type open(int port, int n = 100) {
			if (addr_.assign(port) == false || socktcp::open() < 0) return this->socket();
			if (::bind(this->socket(), addr_.base_data(), addr_.size()) == -1 ||
				::listen(this->socket(), n) == -1) {
				this->close();
			}
			return this->socket();
		}
		
		virtual socktcp accept(socket_address& addr, int retry = 10) {
			socket_type peer = -1;
			typename socket_address::inet_type tmp;
			socklen_type len = sizeof(tmp);
			
			for (int i = 0; i < retry; i++) {
				peer = ::accept(this->socket(), (struct sockaddr*)&tmp, &len);
				if (peer == -1) sleep(1);
				else break;
			}
			
			addr = tmp;
			socktcp dest(peer);
			return dest;
		}
		
		virtual socktcp accept(int retry = 10) {
			socket_address tmp;
			return this->accept(tmp, retry);
		}
	
	private:
		socket_address addr_;
	};
	
	/* --------------------------------------------------------------------- */
	//  basic_socket_tcpclient
	/* --------------------------------------------------------------------- */
	template <
		int Type = SOCK_STREAM,
		int Family = AF_INET
	>
	class basic_socket_tcpclient : public basic_socket<Type, Family> {
	public:
		typedef char char_type;
		typedef basic_socket<Type, Family> socktcp;
		typedef basic_socket_address<Family> socket_address;
		typedef typename std::basic_string<char_type> string_type;
		
		// constructor and destructor
		basic_socket_tcpclient() : socktcp(), addr_() {}
		
		explicit basic_socket_tcpclient(const char_type* host, int port) :
			socktcp(), addr_() { this->connect(host, port); }
		
		explicit basic_socket_tcpclient(const string_type& host, int port) :
			socktcp(), addr_() { this->connect(host, port); }
		
		virtual ~basic_socket_tcpclient() {}
		
		// operator
		virtual socket_type connect(const char_type* host, int port) {
			if (addr_.assign(host, port) == false || this->open() < 0) return -1;
			if (::connect(this->socket(), addr_.base_data(), addr_.size()) == -1) {
				this->close();
				return -1;
			}
			return this->socket();
		}
		
		virtual socket_type connect (const string_type& host, int port) {
			return this->connect(host.c_str(), port);
		}
	
	private:
		socket_address addr_;
	};
	
	typedef basic_socket_tcpserver<> tcpserver;
	typedef basic_socket_tcpclient<> tcpclient;
}

#endif // CLX_SOCKTCP_H
