/* ------------------------------------------------------------------------- */
/*
 *  tokenizer.h
 *
 *  Copyright (c) 2004 - 2006 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Fri 20 Jan 2006 21:19:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TOKENIZER_H
#define CLX_TOKENIZER_H

#include <string>
#include <vector>
#include "tokenizer_func.h"
#include "lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_tokenizer
	/* --------------------------------------------------------------------- */
	template <
		class TokenizerFunc,
		class String = std::string,
		class Container = std::vector<String>
	>
	class basic_tokenizer {
	public:
		typedef String string_type;
		typedef TokenizerFunc func;
		typedef Container container;
		
		typedef typename container::value_type value_type;
		typedef typename container::const_reference const_reference;
		typedef typename container::const_iterator const_iterator;
		typedef typename container::size_type size_type;
		
		// constructor and destructor
		explicit basic_tokenizer(const func& f = func()) :
			f_(f), v_() {}
		
		explicit basic_tokenizer(const string_type& s, const func& f = func()) :
			f_(f), v_() { this->assign(s); }
		
		virtual ~basic_tokenizer() {}
		
		//operator
		virtual void reset() {
			f_.reset();
			v_.clear();
		}
		
		virtual container& assign(const string_type& s) {
			typedef typename string_type::iterator string_iterator;
			
			string_type token;
			string_type tmp(s);
			string_iterator pos = tmp.begin();
			
			this->reset();
			while (f_(pos, tmp.end(), token)) {
				v_.push_back(lexical_cast<value_type>(token));
				if (pos++ == tmp.end()) break;
			}
			return v_;
		}
		
		virtual container& assign(const string_type& s, const func& f) {
			f_ = f;
			return this->assign(s);
		}
		
		virtual container& operator()(const string_type& s) { return this->assign(s); }
		virtual container& operator()(const string_type& s, const func& f) { return this->assign(s, f); }
		
		bool empty() const { return v_.empty(); }
		size_type size() const { return v_.size(); }
		
		// access each token.
		const_reference at(size_type i) const { return v_.at(i); }
		const_reference operator[](size_type i) const { return v_[i]; }
		const_iterator begin() const { return v_.begin(); }
		const_iterator end() const { return v_.end(); }
		
	private:
		func f_;
		container v_;
	};
	
	typedef basic_tokenizer<char_separator<char> > tokenizer;
	typedef basic_tokenizer<escape_separator<char> > csv;
	typedef basic_tokenizer<format_separator<char> > strftokenizer;
}

#endif // CLX_TOKENIZER_H
